# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
#
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------
# pylint: skip-file
# flake8: noqa
from msrest.serialization import Model


class CertificateSet(Model):

    _attribute_map = {
        'certificates': {'key': 'certificates', 'type': '[SecurityDomainCertificateItem]'},
        'required': {'key': 'required', 'type': 'int'}
    }

    def __init__(self, *, certificates=None, required=None, **kwargs) -> None:
        super(CertificateSet, self).__init__(**kwargs)
        self.certificates = certificates
        self.required = required
